from tsuru_dashboard import engine


class AutoScaleTab(engine.Tab):
    name = 'autoscale'
    url_name = 'autoscale-app-info'

try:
    engine.get('app').register_tab(AutoScaleTab)
except engine.AppNotFound:
    pass



from django import template

register = template.Library()


@register.filter
def resource_as_percent(value):
    factor = 100.0
    if value.endswith('m'):
        factor = 0.1
        value = value[0:-1]
    value = float(value) * factor
    return "{}%".format(value)



import json

from django.shortcuts import redirect
from django.core.urlresolvers import reverse
from django.contrib import messages
from django.views.generic import FormView, View
from tsuru_dashboard.apps.views import AppMixin
import requests

from tsuru_autoscale.wizard import client as wclient
from tsuru_autoscale.native import forms


class NativeAutoscale(AppMixin, FormView):
    template_name = "native/index.html"
    form_class = forms.ScaleForm

    def get_form(self):
        token = self.request.session.get('tsuru_token').split(" ")[-1]
        app_name = self.kwargs.get('app_name')
        form = forms.ScaleForm(self.request.POST or None)
        p_list = wclient.process_list(app_name, token)
        form.fields['process'].choices = p_list
        return form

    def get_context_data(self, *args, **kwargs):
        return super(NativeAutoscale, self).get_context_data(*self.args, **self.kwargs)

    def form_valid(self, form):
        token = self.request.session.get('tsuru_token').split(" ")[-1]
        app_name = self.kwargs.get('app_name')
        target_cpu = form.cleaned_data["target_cpu"]
        data = {
            "process": form.cleaned_data["process"],
            "minUnits": form.cleaned_data["min_units"],
            "maxUnits": form.cleaned_data["max_units"],
            "averageCPU": str(target_cpu),
        }

        try:
            add_autoscale(app_name, data, token)
        except Exception as e:
            messages.error(self.request, e)
            return self.form_invalid(form)
        else:
            messages.success(self.request, u"Auto scale saved.")
            url = reverse("autoscale-app-info", args=[app_name])
            return redirect(url)


class NativeAutoscaleRemove(AppMixin, View):
    def get(self, request, app_name, process):
        token = request.session.get('tsuru_token').split(" ")[-1]

        try:
            remove_autoscale(app_name, process, token)
        except Exception as e:
            messages.error(request, e)
        else:
            messages.success(request, u"Auto scale saved.")

        url = reverse("autoscale-app-info", args=[app_name])
        return redirect(url)


def add_autoscale(app, data, token):
    url = "{}/apps/{}/units/autoscale".format(wclient.tsuru_host(), app)
    headers = {
        "Authorization": wclient.clean_token(token),
        "Content-Type": "application/json",
    }
    response = requests.post(url, data=json.dumps(data), headers=headers)
    if response.status_code != 200:
        raise Exception(
            'Invalid response {}: {}'.format(response.status_code, response.text),
        )


def remove_autoscale(app, process, token):
    url = "{}/apps/{}/units/autoscale?process={}".format(wclient.tsuru_host(), app, process)
    headers = {
        "Authorization": wclient.clean_token(token),
        "Content-Type": "application/json",
    }
    response = requests.delete(url, headers=headers)
    if response.status_code != 200:
        raise Exception('invalid response {}: {}'.format(response.status_code, response.text))



from django import forms


class ScaleForm(forms.Form):
    process = forms.ChoiceField(label=u'Process',
                                widget=forms.Select(attrs={'class': 'form-control'}))
    min_units = forms.IntegerField(label=u'Min units', min_value=1, initial=2,
                                   widget=forms.NumberInput(attrs={'class': 'form-control'}))
    max_units = forms.IntegerField(label=u'Max units', min_value=2, initial=2,
                                   widget=forms.NumberInput(attrs={'class': 'form-control'}))
    target_cpu = forms.IntegerField(label=u'Target CPU %', min_value=10, initial=50,
                                    max_value=90,
                                    widget=forms.NumberInput(attrs={'class': 'form-control'}))

    def clean(self):
        cleaned_data = super(ScaleForm, self).clean()
        min_units = cleaned_data.get("min_units")
        max_units = cleaned_data.get("max_units")

        if min_units >= max_units:
            self.add_error('max_units', "Max units should be greater than min units.")



from django import forms


OPERATOR_CHOICES = (
    ('>', '>'),
    ('>=', '>='),
    ('<=', '<='),
    ('<', '<'),
    ('!=', '!='),
)


AGGREGATOR_CHOICES = (
    ('avg', 'average'),
    ('max', 'maximum'),
)


class ScaleForm(forms.Form):
    metric = forms.ChoiceField()
    aggregator = forms.ChoiceField(AGGREGATOR_CHOICES)
    operator = forms.ChoiceField(OPERATOR_CHOICES)
    value = forms.IntegerField(min_value=0)
    step = forms.IntegerField(label=u'Step (in units)', min_value=1, initial=1)
    wait = forms.IntegerField(label=u'Wait time (in seconds)')

    def __init__(self, min_wait, *args, **kwargs):
        super(ScaleForm, self).__init__(*args, **kwargs)
        self.fields['wait'] = forms.IntegerField(label=u'Wait time (in seconds)', min_value=min_wait, initial=min_wait)

    def clean_value(self):
        return str(self.cleaned_data.get('value'))

    def clean_step(self):
        return str(self.cleaned_data.get('step'))


class ConfigForm(forms.Form):
    min = forms.IntegerField(label=u'Start with units', min_value=2, initial=2)
    process = forms.ChoiceField()



from django.shortcuts import render

from tsuru_autoscale.instance import client
from tsuru_autoscale.event import client as eclient


def list(request, app_name=None):
    token = request.session.get('tsuru_token').split(" ")[-1]
    instances = client.list(token).json()
    context = {
        "list": instances,
    }
    return render(request, "instance/list.html", context)


def get(request, name):
    token = request.session.get('tsuru_token').split(" ")[-1]
    instance = client.get(name, token).json()
    alarms = client.alarms_by_instance(name, token).json() or []

    events = []

    for alarm in alarms:
        events.extend(eclient.list(alarm["name"], token).json())

    context = {
        "item": instance,
        "alarms": alarms,
        "events": events,
    }
    return render(request, "instance/get.html", context)



from django.conf.urls import url

from tsuru_autoscale.native import views


urlpatterns = [
    url(r'^(?P<process>[\w-]+)/remove/$', views.NativeAutoscaleRemove.as_view(), name='native-remove'),
    url(r'^$', views.NativeAutoscale.as_view(), name='native-new'),
]



from django.conf.urls import url, include

from tsuru_autoscale.app import views

urlpatterns = [
    url(r'^apps/(?P<app_name>[\w-]+)/autoscale/$', views.AutoscaleApp.as_view(), name='autoscale-app-info'),
    url(r'^apps/(?P<app_name>[\w-]+)/autoscale/native/', include('tsuru_autoscale.native.urls')),
]



import os
import json
import urllib

import requests


def host():
    return os.environ.get("AUTOSCALE_HOST", "")


def tsuru_host():
    return os.environ.get("TSURU_HOST", "")


def clean_token(token):
    if token.lower().startswith("bearer "):
        return token
    token = urllib.unquote(token)
    token = "bearer {}".format(token)
    return token


def app_info(name, token):
    url = "{}/apps/{}".format(tsuru_host(), name)
    headers = {"Authorization": clean_token(token)}
    response = requests.get(url, headers=headers)
    if response.status_code == 200:
        return response.json()
    return None


def pool_info(name, token):
    url = "{}/pools/{}".format(tsuru_host(), name)
    headers = {"Authorization": clean_token(token)}
    response = requests.get(url, headers=headers)
    if response.status_code == 200:
        return response.json()
    return None


def process_list(instance_name, token):
    app = app_info(instance_name, token)
    process = set()

    for u in app.get('units', []):
        process.add(u['ProcessName'])

    p_list = []
    for u in list(process):
        p_list.append((u, u))

    return p_list


def new(data, token):
    url = "{}/wizard".format(host())
    headers = {"Authorization": token}
    response = requests.post(url, data=json.dumps(data), headers=headers)
    return response


def get(name, token):
    url = "{}/wizard/{}".format(host(), name)
    headers = {"Authorization": token}
    response = requests.get(url, headers=headers)
    return response


def remove(name, token):
    url = "{}/wizard/{}".format(host(), name)
    headers = {"Authorization": token}
    response = requests.delete(url, headers=headers)
    return response


def enable(name, token):
    url = "{}/wizard/{}/enable".format(host(), name)
    headers = {"Authorization": token}
    response = requests.post(url, headers=headers)
    return response


def disable(name, token):
    url = "{}/wizard/{}/disable".format(host(), name)
    headers = {"Authorization": token}
    response = requests.post(url, headers=headers)
    return response


def events(name, token):
    url = "{}/wizard/{}/events".format(host(), name)
    headers = {"Authorization": token}
    response = requests.get(url, headers=headers)
    return response



from django.test import TestCase
from django.core.urlresolvers import reverse
from django.conf import settings

from tsuru_autoscale.wizard import forms
from tsuru_autoscale.wizard import client

from importlib import import_module

import mock
import httpretty
import os


class ScaleFormTest(TestCase):
    def test_required_fields(self):
        fields = {
            "metric": True,
            "operator": True,
            "value": True,
            "step": True,
            "wait": True,
            "aggregator": True,
        }

        form = forms.ScaleForm(42)

        for field, required in fields.items():
            self.assertEqual(form.fields[field].required, required)
        self.assertEqual(form.fields['wait'].min_value, 42)


class ConfigFormTest(TestCase):
    def test_required_fields(self):
        fields = {
            "min": True,
        }

        form = forms.ConfigForm()

        for field, required in fields.items():
            self.assertEqual(form.fields[field].required, required)


class IndexTestCase(TestCase):
    def setUp(self):
        settings.SESSION_ENGINE = 'django.contrib.sessions.backends.file'
        engine = import_module(settings.SESSION_ENGINE)
        store = engine.SessionStore()
        store.save()
        self.session = store
        self.client.cookies[settings.SESSION_COOKIE_NAME] = store.session_key
        self.session["tsuru_token"] = "b bla"
        self.session.save()

    @mock.patch("tsuru_autoscale.wizard.client.process_list")
    @mock.patch("tsuru_autoscale.datasource.client.list")
    def test_index(self, dlist_mock, process_mock):
        url = reverse("wizard-new", args=["instance"])
        response = self.client.get(url)
        self.assertTemplateUsed(response, "wizard/index.html")

    @mock.patch("tsuru_autoscale.wizard.client.process_list")
    @mock.patch("tsuru_autoscale.datasource.client.list")
    def test_forms_prefix(self, dlist_mock, process_mock):
        url = reverse("wizard-new", args=["instance"])
        response = self.client.get(url)

        forms = {
            "scale_up_form": "scale_up",
            "scale_down_form": "scale_down",
        }

        for f, prefix in forms.items():
            self.assertEqual(response.context[f].prefix, prefix)

    @mock.patch("tsuru_autoscale.wizard.client.process_list")
    @mock.patch("tsuru_autoscale.datasource.client.list")
    def test_config_process_list(self, dlist_mock, process_mock):
        process = [("web", "web")]
        process_mock.return_value = process

        url = reverse("wizard-new", args=["instance"])
        response = self.client.get(url)

        choices = response.context["config_form"].fields["process"].choices
        self.assertListEqual(process, choices)

    @mock.patch("tsuru_autoscale.wizard.client.process_list")
    @mock.patch("tsuru_autoscale.datasource.client.list")
    def test_scale_metrics(self, dlist_mock, process_mock):
        data = [{"Name": "cpu"}, {"Name": "mem"}]
        response_mock = mock.Mock()
        response_mock.json.return_value = data
        dlist_mock.return_value = response_mock

        url = reverse("wizard-new", args=["instance"])
        response = self.client.get(url)

        expected_choices = [("cpu", "cpu"), ("mem", "mem")]

        choices = response.context["scale_up_form"].fields["metric"].choices
        self.assertListEqual(expected_choices, choices)

        choices = response.context["scale_down_form"].fields["metric"].choices
        self.assertListEqual(expected_choices, choices)


class ClientTestCase(TestCase):
    def setUp(self):
        httpretty.enable()

    def tearDown(self):
        httpretty.disable()
        httpretty.reset()

    def test_new(self):
        os.environ["AUTOSCALE_HOST"] = "http://autoscalehost.com"
        httpretty.register_uri(
            httpretty.POST,
            "http://autoscalehost.com/wizard",
        )

        client.new({}, "token")

    def test_get(self):
        os.environ["AUTOSCALE_HOST"] = "http://autoscalehost.com"
        httpretty.register_uri(
            httpretty.GET,
            "http://autoscalehost.com/wizard/name",
        )

        client.get("name", "token")

    def test_events(self):
        os.environ["AUTOSCALE_HOST"] = "http://autoscalehost.com"
        httpretty.register_uri(
            httpretty.GET,
            "http://autoscalehost.com/wizard/name/events",
        )

        client.events("name", "token")

    def test_remove(self):
        os.environ["AUTOSCALE_HOST"] = "http://autoscalehost.com"
        httpretty.register_uri(
            httpretty.DELETE,
            "http://autoscalehost.com/wizard/name",
        )

        client.remove("name", "token")


class RemoveTestCase(TestCase):
    def setUp(self):
        settings.SESSION_ENGINE = 'django.contrib.sessions.backends.file'
        engine = import_module(settings.SESSION_ENGINE)
        store = engine.SessionStore()
        store.save()
        self.session = store
        self.client.cookies[settings.SESSION_COOKIE_NAME] = store.session_key
        self.session["tsuru_token"] = "b bla"
        self.session.save()

    @mock.patch("tsuru_autoscale.wizard.client.remove")
    def test_remove(self, remove_mock):
        url = reverse("wizard-remove", args=["name"])
        response = self.client.get(url)

        url = reverse("autoscale-app-info", args=["name"])
        self.assertIn(url, response.url)
        remove_mock.assert_called_with("name", "bla")

    @mock.patch("tsuru_autoscale.wizard.client.enable")
    def test_enable(self, enable_mock):
        url = reverse("wizard-enable", args=["name"])
        response = self.client.get(url)

        url = reverse("autoscale-app-info", args=["name"])
        self.assertIn(url, response.url)
        enable_mock.assert_called_with("name", "bla")

    @mock.patch("tsuru_autoscale.wizard.client.disable")
    def test_disable(self, disable_mock):
        url = reverse("wizard-disable", args=["name"])
        response = self.client.get(url)

        url = reverse("autoscale-app-info", args=["name"])
        self.assertIn(url, response.url)
        disable_mock.assert_called_with("name", "bla")



from django.test import TestCase
from django.core.urlresolvers import reverse
from django.conf import settings

from importlib import import_module

import mock


class IndexTestCase(TestCase):
    def setUp(self):
        settings.SESSION_ENGINE = 'django.contrib.sessions.backends.file'
        engine = import_module(settings.SESSION_ENGINE)
        store = engine.SessionStore()
        store.save()
        self.session = store
        self.client.cookies[settings.SESSION_COOKIE_NAME] = store.session_key
        self.session["tsuru_token"] = "b bla"
        self.session.save()

    @mock.patch("tsuru_dashboard.auth.views.token_is_valid")
    @mock.patch("tsuru_dashboard.apps.views.AppMixin.get_app")
    @mock.patch("tsuru_autoscale.instance.client.list")
    def test_index(self, list_mock, get_app_mock, token_is_valid):
        get_app_mock.return_value = {"name": "myapp"}
        token_is_valid.return_value = True

        url = "{}".format(reverse("autoscale-app-info", args=["app"]))
        response = self.client.get(url)
        self.assertTemplateUsed(response, "app/index.html")

    @mock.patch("tsuru_dashboard.auth.views.token_is_valid")
    @mock.patch("tsuru_dashboard.apps.views.AppMixin.get_app")
    @mock.patch("tsuru_autoscale.instance.client.list")
    def test_index_instance_not_found(self, list_mock, get_app_mock, token_is_valid):
        get_app_mock.return_value = {"name": "myapp"}
        token_is_valid.return_value = True

        response_mock = mock.Mock()
        response_mock.json.return_value = None
        list_mock.return_value = response_mock

        url = "{}".format(reverse("autoscale-app-info", args=["app"]))
        response = self.client.get(url)

        self.assertTemplateUsed(response, "app/index.html")



from django.http import HttpResponse


class AuthMiddleware(object):
    def process_request(self, request):
        token = request.GET.get("TSURU_TOKEN")

        if hasattr(request, 'session') and not token:
            token = request.session.get('tsuru_token')

        if not token:
            return HttpResponse(status=401)

        request.token = token



from django import forms


class ActionForm(forms.Form):
    name = forms.CharField()
    url = forms.CharField()
    method = forms.CharField()
    body = forms.CharField(required=False)
    headers = forms.CharField(required=False)



from django.conf.urls import url

from tsuru_autoscale.alarm import views
from tsuru_autoscale.event.views import list as event_list

urlpatterns = [
    url(r'^$', views.list, name='alarm-list'),
    url(r'^new/$', views.new, name='alarm-new'),
    url(r'^(?P<name>[\w\s-]+)/remove/$', views.remove, name='alarm-remove'),
    url(r'^(?P<alarm_name>[\w\s-]+)/event/$', event_list, name='event-list'),
    url(r'^(?P<name>[\w\s-]+)/$', views.get, name='alarm-get'),
]



import os
import logging

import requests


def host():
    return os.environ.get("AUTOSCALE_HOST", "")


def list(token):
    url = "{}/service/instance".format(host())
    headers = {"Authorization": token}
    logging.debug("trying to get service instances - {}".format(url))
    response = requests.get(url, headers=headers)
    logging.debug("service instances response - {}".format(response))
    return response


def get(name, token):
    url = "{}/service/instance/{}".format(host(), name)
    headers = {"Authorization": token}
    response = requests.get(url, headers=headers)
    return response


def alarms_by_instance(instance, token):
    url = "{}/alarm/instance/{}".format(host(), instance)
    headers = {"Authorization": token}
    response = requests.get(url, headers=headers)
    return response



from django.conf.urls import url

from tsuru_autoscale.action import views


urlpatterns = [
    url(r'^new/$', views.new, name='action-new'),
    url(r'^(?P<name>[\w\s-]+)/remove/$', views.remove, name='action-remove'),
    url(r'^(?P<name>[\w\s-]+)/$', views.get, name='action-get'),
    url(r'^$', views.list, name='action-list'),
]



from django.shortcuts import render, redirect
from django.core.urlresolvers import reverse
from django.contrib import messages

from tsuru_autoscale.datasource.forms import DataSourceForm
from tsuru_autoscale.datasource import client


def new(request):
    form = DataSourceForm(request.POST or None)

    if form.is_valid():
        token = request.GET.get("TSURU_TOKEN")
        response = client.new(form.cleaned_data, token)
        if response.status_code > 399:
            messages.error(request, response.text)
        else:
            messages.success(request, u"Data source saved.")
        url = "{}?TSURU_TOKEN={}".format(reverse('datasource-list'), token)
        return redirect(url)

    context = {"form": form}
    return render(request, "datasource/new.html", context)


def list(request):
    token = request.GET.get("TSURU_TOKEN")
    datasources = client.list(token).json()
    context = {
        "list": datasources,
    }
    return render(request, "datasource/list.html", context)


def remove(request, name):
    token = request.GET.get("TSURU_TOKEN")
    client.remove(name, token)
    messages.success(request, u"Data source  {} remove.".format(name))
    url = "{}?TSURU_TOKEN={}".format(reverse('datasource-list'), token)
    return redirect(url)


def get(request, name):
    token = request.GET.get("TSURU_TOKEN")
    datasource = client.get(name, token).json()
    context = {
        "item": datasource,
    }
    return render(request, "datasource/get.html", context)



from django.test import TestCase
from django.core.urlresolvers import reverse

from tsuru_autoscale.event import client

import httpretty
import mock

import os


class ListTestCase(TestCase):
    @mock.patch("tsuru_autoscale.event.client.list")
    def test_list(self, list_mock):
        url = "{}?TSURU_TOKEN=bla".format(reverse("event-list", args=["alarm_name"]))
        response = self.client.get(url)

        self.assertTemplateUsed(response, "event/list.html")
        self.assertIn('list', response.context)
        list_mock.assert_called_with("alarm_name", "bla")


class ClientTestCase(TestCase):
    def setUp(self):
        httpretty.enable()

    def tearDown(self):
        httpretty.disable()
        httpretty.reset()

    def test_list(self):
        os.environ["AUTOSCALE_HOST"] = "http://autoscalehost.com"
        httpretty.register_uri(
            httpretty.GET,
            "http://autoscalehost.com/alarm/abc/event",
        )

        client.list("abc", "token")



import os

from django.views.generic import TemplateView

from tsuru_autoscale.instance import client
from tsuru_autoscale.wizard import client as wclient
from tsuru_dashboard.apps.views import AppMixin


class AutoscaleApp(AppMixin, TemplateView):
    template_name = 'app/index.html'

    def get_context_data(self, *args, **kwargs):
        context = super(AutoscaleApp, self).get_context_data(*args, **kwargs)

        token = self.request.session.get('tsuru_token').split(" ")[-1]
        instances = client.list(token).json() or []

        app = context.get('app', {})
        pool_name = app.get('pool')
        pool_info = wclient.pool_info(pool_name, token) if pool_name else None

        provisioner = pool_info.get('provisioner') if pool_info else None

        native_disable = os.environ.get("AUTOSCALE_NATIVE_DISABLE") in ["True", "1", "true"]
        supports_native = provisioner == "kubernetes" and not native_disable

        instance = None
        auto_scale = None
        events = None
        legacy = self.request.GET.get('legacy')

        for inst in instances:
            if app.get('name') in inst.get('Apps', []):
                instance = inst

                response = wclient.get(instance["Name"], token)
                if response.status_code == 200:
                    auto_scale = response.json()
                    events = wclient.events(instance["Name"], token).json()

        context.update({
            "instance": instance,
            "auto_scale": auto_scale,
            "events": events,
            "supports_native": supports_native,
            "is_legacy": legacy == "1" or legacy == "true" or legacy == "True" or not supports_native,
        })
        return context
